package org.dusystems.oobiblio;

import static org.dusystems.oobiblio.OOBibConstants.bib_types;
import static org.dusystems.oobiblio.OOBibConstants.db_attrs;
import org.kobjects.jdbc.TableManager;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.sql.ResultSet;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * OpenOffice Bibliography Fix.
 *
 * @author omikheev
 * @version $Id: OOBF.java,v 1.5 2009/01/14 19:20:09 cvsmihel Exp $
 */
public class OOBF {

    public static boolean debug;

	public static void main(String[] args) throws Exception {
		if (args.length < 2) {
			System.out.println("java -jar oobf.jar [options] arg1 arg2 [encoding]");
			System.out.println("arg1: odt file");
			System.out.println("arg2: dbf file");
			System.out.println("encoding defaults to UTF8");
			System.out.println("options:");
			System.out.println("    -e - erase bibliography mark if match not found in DBF");
			System.out.println("    -d - debug");
			System.exit(2);
		}
		int i = 0;
		boolean bErase = false;
		if(args[0].startsWith("-")) {
			if("-e".equals(args[0])) {
                bErase = true;
            } else if("-d".equals(args[0])) {
                debug = true;
            }
            i++;
		}
		String sOdt = args[i++];
		String sDbf = args[i++];
		File f = new File(sOdt);
		if (!f.exists()) {
			System.out.println("File doesn't exist");
			System.exit(2);
		}
		Properties prop = new Properties();
		prop.setProperty("encoding", args.length > i ? args[i] : "UTF8");
		System.out.println("Using encoding: " + prop.getProperty("encoding"));
		ResultSet rs = TableManager.getResultSet("dbase", sDbf, prop, TableManager.READ);
		List lstIds = new ArrayList();
		List<Map<String, String>> lstVals = new ArrayList<Map<String, String>>();
		while (rs.next()) {
			lstIds.add(rs.getString("identifier").trim());
			Map mpVals = new HashMap();
			for (String attrName : db_attrs) {
				System.out.println("attrName = " + attrName);
				String sAttrVal = null;
				try {
					sAttrVal = rs.getString(attrName);
				} catch (Exception e) {
				}
				if(sAttrVal == null || sAttrVal.length() == 0) {
					if(debug) System.err.println(attrName + " was not found in database");
					continue;
				}
				if (!"type".equals(attrName)) {
					mpVals.put(attrName, sAttrVal.trim());
				} else {
					mpVals.put(attrName, bib_types[new Integer(sAttrVal.trim())]);
				}
			}
			lstVals.add(mpVals);
		}

		XMLReader parser = XMLReaderFactory.createXMLReader();
		parser.setFeature("http://xml.org/sax/features/namespaces", true);
		parser.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
		OOSaxHandler handler = new OOSaxHandler(bErase);
		handler.setDbVals(lstVals);
		handler.setIds(lstIds);
		File fTempDir = new File("tmp" + System.currentTimeMillis());
		fTempDir.mkdir();
		File fTemp = new File(fTempDir.getAbsolutePath() + "/content.xml");
		handler.setOutput(new FileOutputStream(fTemp), "UTF8");
		parser.setContentHandler(handler);
		parser.setErrorHandler(handler);
		InputStream istr = (new ZipFile(f)).getInputStream(new ZipEntry("content.xml"));
		InputSource isrc = new InputSource(istr);
		parser.parse(isrc);
		System.out.println("File created: " + fTemp.getAbsolutePath());
		JarUpdater.updateJar(sOdt, fTempDir.getAbsolutePath() + "/content.xml");
		System.out.println("Done!");
		fTemp.delete();
		fTempDir.delete();
	}

}
