package org.dusystems.oobiblio;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.DefaultHandler;
import static org.dusystems.oobiblio.OOBibConstants.*;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

/**
 * Based on a sample SAX2 writer.
 *
 * @author Oleg Mikheev
 * @author Andy Clark, IBM
 * @version $Id: OOSaxHandler.java,v 1.6 2009/01/14 19:20:08 cvsmihel Exp $
 */
public class OOSaxHandler extends DefaultHandler implements LexicalHandler {

	protected PrintWriter fOut;
	protected boolean fCanonical;
	protected int fElementDepth;
	protected Locator fLocator;
	protected boolean fXML11;
	protected boolean fInCDATA;
	private boolean eraseNotMatched;
	private boolean ignoreBiblioClose;

	public OOSaxHandler() {
	} // <init>()

	public OOSaxHandler(boolean eraseNotMatched) {
		this.eraseNotMatched = eraseNotMatched;
	}
//
	// Public methods
	//

	public void setCanonical(boolean canonical) {
		fCanonical = canonical;
	} // setCanonical(boolean)

	public void setOutput(OutputStream stream, String encoding)
			throws UnsupportedEncodingException {

		if (encoding == null) {
			encoding = "UTF8";
		}

		java.io.Writer writer = new OutputStreamWriter(stream, encoding);
		fOut = new PrintWriter(writer);

	} // setOutput(OutputStream,String)

	public void setOutput(java.io.Writer writer) {

		fOut = writer instanceof PrintWriter
				? (PrintWriter) writer : new PrintWriter(writer);

	} // setOutput(java.io.OOSaxHandler)

	//
	// ContentHandler methods
	//

	/**
	 * Set Document Locator.
	 */
	public void setDocumentLocator(Locator locator) {
		fLocator = locator;
	} // setDocumentLocator(Locator)

	/**
	 * Start document.
	 */
	public void startDocument() throws SAXException {

		fElementDepth = 0;
		fXML11 = false;
		fInCDATA = false;

	} // startDocument()

	/**
	 * Processing instruction.
	 */
	public void processingInstruction(String target, String data)
			throws SAXException {

		if (fElementDepth > 0) {
			fOut.print("<?");
			fOut.print(target);
			if (data != null && data.length() > 0) {
				fOut.print(' ');
				fOut.print(data);
			}
			fOut.print("?>");
			fOut.flush();
		}

	} // processingInstruction(String,String)

	/**
	 * Start element.
	 */
	public void startElement(String uri, String local, String raw,
							 Attributes attrs) throws SAXException {

		// Root Element
		if (fElementDepth == 0) {
			if (fLocator != null) {
				fXML11 = "1.1".equals(getVersion());
				fLocator = null;
			}

			// The XML declaration cannot be printed in startDocument because
			// the version reported by the Locator cannot be relied on until after
			// the XML declaration in the instance document has been read.
			if (!fCanonical) {
				if (fXML11) {
					fOut.println("<?xml version=\"1.1\" encoding=\"UTF-8\"?>");
				} else {
					fOut.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
				}
				fOut.flush();
			}
		}

		boolean bBiblio = false;
		Map mpAttrs = null;
		if ("text:bibliography-mark".equals(raw)) {
			bBiblio = true;
			mpAttrs = new HashMap<String, String>();
		}
		if (bBiblio && attrs != null) {
			int len = attrs.getLength();
			for (int i = 0; i < len; i++) {
				mpAttrs.put(attrs.getQName(i), attrs.getValue(i));
			}
            Object oBibId = mpAttrs.get("text:identifier");
            if(OOBF.debug) System.out.println("Found text:bibliography-mark for " + oBibId);
			int idx = ids.indexOf(oBibId);
			if (idx == -1) {
				if(eraseNotMatched) {
					System.out.println("Erasing bibliography mark not found in DBF: " + oBibId);
					ignoreBiblioClose = true;
					return;
				} else {
					System.out.println("Bibliography mark not found in DBF: " + oBibId + ". Use -e to remove from ODT.");
					fElementDepth++;
					fOut.print('<');
					fOut.print(raw);
					printAttrs(attrs);
				}
			} else {
				fElementDepth++;
				fOut.print('<');
				fOut.print(raw);
				for (int j = 0; j < db_attrs.length; j++) {
					String sDbValue = dbVals.get(idx).get(db_attrs[j]);
					if(sDbValue == null) continue;
					sDbValue = sDbValue.trim();
					Object oOdtValue = mpAttrs.get(odt_attrs[j]);
					oOdtValue = oOdtValue == null ? "" : ((String) oOdtValue).trim();
					if (!sDbValue.equals(oOdtValue)) {
						System.out.println("Updating "+db_attrs[j]+": {" + sDbValue + "} -> {" + oOdtValue + "}");
					}
					if (sDbValue.length() > 0) {
						fOut.print(' ');
						fOut.print(odt_attrs[j]);
						fOut.print("=\"");
						normalizeAndPrint(sDbValue, true);
						fOut.print('"');
					}
				}
			}
		} else if(attrs != null && !bBiblio) {
			fElementDepth++;
			fOut.print('<');
			fOut.print(raw);
			printAttrs(attrs);
		}
		fOut.print('>');
		fOut.flush();

	} // startElement(String,String,String,Attributes)

	private void printAttrs(Attributes attrs) {
		int len = attrs.getLength();
		for (int i = 0; i < len; i++) {
			fOut.print(' ');
			fOut.print(attrs.getQName(i));
			fOut.print("=\"");
			normalizeAndPrint(attrs.getValue(i), true);
			fOut.print('"');
		}
	}

	/**
	 * Characters.
	 */
	public void characters(char ch[], int start, int length)
			throws SAXException {

		if (!fInCDATA) {
			normalizeAndPrint(ch, start, length, false);
		} else {
			for (int i = 0; i < length; ++i) {
				fOut.print(ch[start + i]);
			}
		}
		fOut.flush();

	} // characters(char[],int,int);

	/**
	 * Ignorable whitespace.
	 */
	public void ignorableWhitespace(char ch[], int start, int length)
			throws SAXException {

		characters(ch, start, length);
		fOut.flush();

	} // ignorableWhitespace(char[],int,int);

	/**
	 * End element.
	 */
	public void endElement(String uri, String local, String raw)
			throws SAXException {
		if(ignoreBiblioClose) {
			ignoreBiblioClose = false;
		} else {
			fElementDepth--;
			fOut.print("</");
			fOut.print(raw);
			fOut.print('>');
			fOut.flush();
		}
	} // endElement(String)

	//
	// ErrorHandler methods
	//

	/**
	 * Warning.
	 */
	public void warning(SAXParseException ex) throws SAXException {
		printError("Warning", ex);
	} // warning(SAXParseException)

	/**
	 * Error.
	 */
	public void error(SAXParseException ex) throws SAXException {
		printError("Error", ex);
	} // error(SAXParseException)

	/**
	 * Fatal error.
	 */
	public void fatalError(SAXParseException ex) throws SAXException {
		printError("Fatal Error", ex);
		throw ex;
	} // fatalError(SAXParseException)

	//
	// LexicalHandler methods
	//

	/**
	 * Start DTD.
	 */
	public void startDTD(String name, String publicId, String systemId)
			throws SAXException {
	} // startDTD(String,String,String)

	/**
	 * End DTD.
	 */
	public void endDTD() throws SAXException {
	} // endDTD()

	/**
	 * Start entity.
	 */
	public void startEntity(String name) throws SAXException {
	} // startEntity(String)

	/**
	 * End entity.
	 */
	public void endEntity(String name) throws SAXException {
	} // endEntity(String)

	/**
	 * Start CDATA section.
	 */
	public void startCDATA() throws SAXException {
		if (!fCanonical) {
			fOut.print("<![CDATA[");
			fInCDATA = true;
		}
	} // startCDATA()

	/**
	 * End CDATA section.
	 */
	public void endCDATA() throws SAXException {
		if (!fCanonical) {
			fInCDATA = false;
			fOut.print("]]>");
		}
	} // endCDATA()

	/**
	 * Comment.
	 */
	public void comment(char ch[], int start, int length) throws SAXException {
		if (!fCanonical && fElementDepth > 0) {
			fOut.print("<!--");
			for (int i = 0; i < length; ++i) {
				fOut.print(ch[start + i]);
			}
			fOut.print("-->");
			fOut.flush();
		}
	} // comment(char[],int,int)

	//
	// Protected methods
	//

	/**
	 * Normalizes and prints the given string.
	 */
	protected void normalizeAndPrint(String s, boolean isAttValue) {

		int len = (s != null) ? s.length() : 0;
		for (int i = 0; i < len; i++) {
			char c = s.charAt(i);
			normalizeAndPrint(c, isAttValue);
		}

	} // normalizeAndPrint(String,boolean)

	/**
	 * Normalizes and prints the given array of characters.
	 */
	protected void normalizeAndPrint(char[] ch, int offset, int length, boolean isAttValue) {
		for (int i = 0; i < length; i++) {
			normalizeAndPrint(ch[offset + i], isAttValue);
		}
	} // normalizeAndPrint(char[],int,int,boolean)

	/**
	 * Normalizes and print the given character.
	 */
	protected void normalizeAndPrint(char c, boolean isAttValue) {

		switch (c) {
			case '<': {
				fOut.print("&lt;");
				break;
			}
			case '>': {
				fOut.print("&gt;");
				break;
			}
			case '&': {
				fOut.print("&amp;");
				break;
			}
			case '"': {
				// A '"' that appears in character data
				// does not need to be escaped.
				if (isAttValue) {
					fOut.print("&quot;");
				} else {
					fOut.print("\"");
				}
				break;
			}
			case '\r': {
				// If CR is part of the document's content, it
				// must not be printed as a literal otherwise
				// it would be normalized to LF when the document
				// is reparsed.
				fOut.print("&#xD;");
				break;
			}
			case '\n': {
				if (fCanonical) {
					fOut.print("&#xA;");
					break;
				}
				// else, default print char
			}
			default: {
				// In XML 1.1, control chars in the ranges [#x1-#x1F, #x7F-#x9F] must be escaped.
				//
				// Escape space characters that would be normalized to #x20 in attribute values
				// when the document is reparsed.
				//
				// Escape NEL (0x85) and LSEP (0x2028) that appear in content
				// if the document is XML 1.1, since they would be normalized to LF
				// when the document is reparsed.
				if (fXML11 && ((c >= 0x01 && c <= 0x1F && c != 0x09 && c != 0x0A)
						|| (c >= 0x7F && c <= 0x9F) || c == 0x2028)
						|| isAttValue && (c == 0x09 || c == 0x0A)) {
					fOut.print("&#x");
					fOut.print(Integer.toHexString(c).toUpperCase());
					fOut.print(";");
				} else {
					fOut.print(c);
				}
			}
		}
	} // normalizeAndPrint(char,boolean)

	/**
	 * Prints the error message.
	 */
	protected void printError(String type, SAXParseException ex) {

		System.err.print("[");
		System.err.print(type);
		System.err.print("] ");
		String systemId = ex.getSystemId();
		if (systemId != null) {
			int index = systemId.lastIndexOf('/');
			if (index != -1)
				systemId = systemId.substring(index + 1);
			System.err.print(systemId);
		}
		System.err.print(':');
		System.err.print(ex.getLineNumber());
		System.err.print(':');
		System.err.print(ex.getColumnNumber());
		System.err.print(": ");
		System.err.print(ex.getMessage());
		System.err.println();
		System.err.flush();

	} // printError(String,SAXParseException)

	/**
	 * Extracts the XML version from the Locator.
	 */
	protected String getVersion() {
		if (fLocator == null) {
			return null;
		}
		String version = null;
		Method getXMLVersion = null;
		try {
			getXMLVersion = fLocator.getClass().getMethod("getXMLVersion", new Class[]{});
			// If Locator implements Locator2, this method will exist.
			if (getXMLVersion != null) {
				version = (String) getXMLVersion.invoke(fLocator, (Object[]) null);
			}
		}
		catch (Exception e) {
			// Either this locator object doesn't have
			// this method, or we're on an old JDK.
		}
		return version;
	}

	public List getIds() {
		return ids;
	}

	public void setIds(List ids) {
		this.ids = ids;
	}

	public List<Map<String, String>> getDbVals() {
		return dbVals;
	}

	public void setDbVals(List<Map<String, String>> dbVals) {
		this.dbVals = dbVals;
	}

	private List ids;
	private List<Map<String, String>> dbVals;
}
