package org.kobjects.jdbc;

// TODO: reactivate BOOLEAN(!!!)

import java.sql.*;


/**
 * Describes a field in a table.
 */
public class Column {


	/**
	 * Holds the field's name.
	 */
	private String name;

	/**
	 * Holds the field's type.
	 */
	private int type;


	/**
	 * Holds the column number. When the colum number is set, the column is
	 * locked and cannot be changed any more.
	 */

	private int number;

	/**
	 * Holds the field's label. This is only interpreted in the UI. If the label
	 * is null, the field's name is displayed instead.
	 */
	private String label;

	private Object[] values;
	private ColumnSet columnSet;

	private boolean autoIncrement;

	private int scale;
	private int precision;

	private int displaySize;

	/**
	 * Creates a new field.  From a user's perspective, new fields are
	 * created using the table's <code>addField()</code> factory
	 * method.
	 */
	public Column(String name, int type) {
		this.name = name;
		this.type = type;
	}

	/**
	 * Create a new field based on result set meta data
	 */

	public Column(ResultSetMetaData meta, int index) throws SQLException {
		this(meta.getColumnName(index), meta.getColumnType(index));
		autoIncrement = meta.isAutoIncrement(index);

		setDisplaySize(meta.getColumnDisplaySize(index));
		setPrecision(meta.getPrecision(index));
		setScale(meta.getScale(index));

		System.out.println("dbcolumn created: " + toString());
	}

	/**
	 * Copy constructor; creates an unlocked copy of the given
	 * Column object.
	 *
	 * @param column
	 */

	public Column(Column column) {
		this.name = column.name;
		this.type = column.type;
	}


	/**
	 * Returns the field's name.
	 */
	public String getName() {
		return name;
	}

	public void setValues(Object[] values) {
		if (number != 0)
			throw new RuntimeException("Column is locked already");
		this.values = values;
	}


	public ColumnSet getColumnSet() {
		return columnSet;
	}

	public Object[] getValues() {
		return values;
	}

	/**
	 * Returns the field's type.
	 */
	public int getType() {
		return type;
	}

	public Class getJavaType() {
		switch (type) {
//			case Types.ARRAY: return "ARRAY";
//			case Types.BIGINT: 
//			case Types.BINARY: return "BINARY";
			case Types.BIT:
			case Types.BOOLEAN:
				return Boolean.class;
			case Types.BLOB:
				return Blob.class;
			case Types.CHAR:
				return String.class;
			case Types.CLOB:
				return Clob.class;
//			case Types.DATALINK: return "DATALINK";
			case Types.DATE:
				return java.sql.Date.class;
//			case Types.DECIMAL: return "DECIMAL";
			case Types.DOUBLE:
				return Double.class;
			case Types.FLOAT:
				return Float.class;
			case Types.INTEGER:
				return Integer.class;
				//		case Types.JAVA_OBJECT: return "JAVA_OBJECT";
				//		case Types.LONGVARBINARY: return "LONGVARBINARY";
			case Types.LONGVARCHAR:
				return String.class;
//			case Types.NUMERIC: return "NUMERIC";
//			case Types.OTHER: return "OTHER";
//			case Types.REAL: return "REAL";
//			case Types.REF: return "REF";
//			case Types.SMALLINT: return "SMALLINT";
//			case Types.STRUCT: return "STRUCT";
			case Types.TIME:
				return Time.class;
			case Types.TIMESTAMP:
				return Timestamp.class;
//			case Types.TINYINT: return "TINYINT";
//			case Types.VARBINARY: return "VARBINARY";
			case Types.VARCHAR:
				return String.class;
			default:
				throw new RuntimeException("java type determ NYI for " + getTypeName());
		}
	}


	public String getTypeName() {
		switch (type) {
			case Types.ARRAY:
				return "ARRAY";
			case Types.BIGINT:
			case Types.BINARY:
				return "BINARY";
			case Types.BOOLEAN:
				return "BOOLEAN";
			case Types.BIT:
				return "BIT";
			case Types.BLOB:
				return "BLOB";
			case Types.CHAR:
				return "CHAR";
			case Types.CLOB:
				return "CLOB";
			case Types.DATALINK:
				return "DATALINK";
			case Types.DATE:
				return "DATE";
			case Types.DECIMAL:
				return "DECIMAL";
			case Types.DOUBLE:
				return "DOUBLE";
			case Types.FLOAT:
				return "FLOAT";
			case Types.INTEGER:
				return "INTEGER";
			case Types.JAVA_OBJECT:
				return "JAVA_OBJECT";
			case Types.LONGVARBINARY:
				return "LONGVARBINARY";
			case Types.LONGVARCHAR:
				return "LONGVARCHAR";
			case Types.NUMERIC:
				return "NUMERIC";
			case Types.OTHER:
				return "OTHER";
			case Types.REAL:
				return "REAL";
			case Types.REF:
				return "REF";
			case Types.SMALLINT:
				return "SMALLINT";
			case Types.STRUCT:
				return "STRUCT";
			case Types.TIME:
				return "TIME";
			case Types.TIMESTAMP:
				return "TIMESTAMP";
			case Types.TINYINT:
				return "TINYINT";
			case Types.VARBINARY:
				return "VARBINARY";
			case Types.VARCHAR:
				return "VARCHAR";
			default:
				throw new RuntimeException("illegal type " + type);
		}
	}


	/**
	 * Returns the field's label.
	 */
	public String getLabel() {
		return label;
	}


	public int getNumber() {
		return number;
	}

	/*
		 * Utility method to compare two field values. The type of the values must
		 * be specified. The method returns a value smaller than zero if value1 < value2,
		 * a value greater than zero if if value1 > value2, and zero otherwiese.
		 * Binary values are compared according to their length, just to have a
		 * defined behaviour for all types.

		public static int compare(int type, Object value1, Object value2) {
		   switch (type) {
				case Column.INTEGER:
				case Column.BITSET: {
					return ((Integer)value1).intValue() - ((Integer)value2).intValue();
				}

				case Column.LONG:
				case Column.DATETIME: {
					long l = ((Long)value1).longValue() - ((Long)value2).longValue();

					if (l < 0) return -1; else if (l > 0) return 1; else return 0;
				}

				case Column.STRING:
				case Column.BOOLEAN: {
					return value1.toString().compareTo(value2.toString());
				}

				case Column.BINARY:
				case Column.GRAPHICS:
					return ((byte[])value1).length - ((byte[])value2).length;

				default: {
					throw new IllegalArgumentException("Illegal type \"" + type + "\"");
				}
			}
		}*/


	public void lock(ColumnSet columnSet) {
		if (this.columnSet != null || this.number != 0) {
			throw new RuntimeException
					("Cannot lock Colum " + this + " for " + columnSet
							+ "; colum is already owned by " + this.columnSet);
		}
		this.columnSet = columnSet;
		this.number = columnSet.getColumnCount();
	}

	/**
	 * Method getDisplaySize.
	 *
	 * @return int
	 */
	public int getDisplaySize() {
		return displaySize;
	}


	public boolean isAutoIncrement() {
		return autoIncrement;
	}


	/**
	 * Sets the displaySize.
	 *
	 * @param displaySize The displaySize to set
	 */
	public void setDisplaySize(int displaySize) {
		this.displaySize = displaySize;
	}

	/**
	 * @return int
	 */
	public int getPrecision() {
		return precision;
	}

	/**
	 * @return int
	 */
	public int getScale() {
		return scale;
	}


	public void setAutoIncrement(boolean ai) {
		this.autoIncrement = ai;
	}

	/**
	 * Sets the precision.
	 *
	 * @param precision The precision to set
	 */
	public void setPrecision(int precision) {
		this.precision = precision;
	}

	/**
	 * Sets the scale.
	 *
	 * @param scale The scale to set
	 */
	public void setScale(int scale) {
		this.scale = scale;
	}


	public String toString() {
		return "#" + number + " " + name + ": " + getTypeName() + "; " + precision + ", " + scale + (autoIncrement ? "(autoInc)  " : "");
	}


	public Object fromString(String value) {
		switch (getType()) {
			case Types.CHAR:
			case Types.VARCHAR:
			case Types.LONGVARCHAR:
				return value;
			case Types.BOOLEAN:
			case Types.BIT:
				return new Boolean(value);
			case Types.REAL:
			case Types.DECIMAL:
			case Types.DOUBLE:
				return new Double(value);
			case Types.FLOAT:
				return new Float(value);
			case Types.INTEGER:
				return new Integer(value);
			case Types.NUMERIC:
				return scale == 0
						? (Object) new Long(value)
						: new Double(value);

			case Types.DATE:
				if (value == null || value.trim().length() == 0)
					return null;
				try {
					return java.sql.Date.valueOf(value);
				}
				catch (Exception e) {
					throw new RuntimeException("Date format errot for value '" + value + "': " + e);
				}
			default:
				throw new RuntimeException(
						"unsupported field type:" + getType());
		}


	}

}
     

