package org.kobjects.jdbc;

import java.sql.*;
import java.util.Hashtable;
import java.util.Vector;

/**
 * @author Stefan Haustein
 */
public class ColumnSet implements ResultSetMetaData {

	private Hashtable names;
	private Vector columns = new Vector();
	private String name;
	private boolean sealed;

	public ColumnSet() {
	}

	public ColumnSet(ResultSetMetaData meta) throws SQLException {
		for (int i = 1; i < meta.getColumnCount(); i++)
			addColumn(new Column(meta, i));

	}


	public int addColumn(Column column) {
		if (sealed) throw new RuntimeException("ColumnSet is locked already!");
		names = null;
		columns.addElement(column);
		column.lock(this);
		return columns.size();
	}


	public Column getColumn(int index) {
		return (Column) columns.elementAt(index - 1);
	}

	public Column getColumn(String name) throws SQLException {
		return getColumn(findColumn(name));
	}


	public int getColumnCount() {
		return columns.size();
	}

	public boolean isAutoIncrement(int column)
			throws SQLException {
		return getColumn(column).isAutoIncrement();
	}


	public boolean isCaseSensitive(int column)
			throws SQLException {
		throw new SQLException("not supported!");
	}


	public boolean isSearchable(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	public boolean isCurrency(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	public int isNullable(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	public boolean isSigned(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	/**
	 * When a Columset is sealed, it becomes immutable
	 */

	public void seal() {
		sealed = true;
	}


	public int getColumnDisplaySize(int column) {
		return getColumn(column).getDisplaySize();
	}


	public String getColumnLabel(int column)
			throws SQLException {
		return getColumn(column).getLabel();
	}


	public String getColumnName(int column)
			throws SQLException {
		return getColumn(column).getName();
	}


	public String getSchemaName(int column)
			throws SQLException {
		throw new SQLException("not supported!");
	}


	public int getPrecision(int column) throws SQLException {
		return getColumn(column).getPrecision();
	}


	public int getScale(int column) throws SQLException {
		return getColumn(column).getScale();
	}


	public String getTableName(int column) throws SQLException {
		return name;
	}


	public String getCatalogName(int column)
			throws SQLException {
		throw new SQLException("not supported!");
	}


	public int getColumnType(int column) throws SQLException {
		return getColumn(column).getType();
	}


	public String getColumnTypeName(int column) {
		return getColumn(column).getTypeName();
	}


	public boolean isReadOnly(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	public boolean isWritable(int column) throws SQLException {
		throw new SQLException("not supported!");
	}


	public boolean isDefinitelyWritable(int column)
			throws SQLException {
		throw new SQLException("not supported!");
	}


	public String getColumnClassName(int column)
			throws SQLException {
		throw new SQLException("not supported!");
	}


	public int findColumn(String columnName) throws SQLException {
		if (names == null) {
			names = new Hashtable();
			for (int i = 1; i <= getColumnCount(); i++) {
				names.put(
						getColumn(i).getName(),
						new Integer(i));
			}

			for (int i = 1; i <= getColumnCount(); i++) {
				if (names.get(
						getColumn(i).getName().toUpperCase()) == null)
					names.put(getColumn(i).getName().toUpperCase(), new Integer(i));
			}
		}

		Integer col = (Integer) names.get(columnName);
		if (col == null) col = (Integer) names.get(columnName.toUpperCase());
		if (col == null)
			throw new SQLException("Column '" + columnName + "' does not exist; existing colums: " + names);
		return col.intValue();

	}

	public <T> T unwrap(Class<T> iface) throws SQLException {
		return null;
	}

	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return false;
	}
}
