/*
 * Created on Jul 24, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.kobjects.jdbc;

import java.sql.SQLException;
import java.util.Comparator;
import java.util.Hashtable;

/**
 *
 */


public class Order implements Comparator {

	public static final int DEFAULT = 0;
	public static final int ASCENDING = 0;
	public static final int DESCENDING = 1;

	Column column;
	int modifier;
	Order remainder;

	/**
	 * Used to invert the comparator result in case of descending order
	 */

	int comparisonFactor;

	public Order(Column column, int modifier, Order remainder) {
		this.column = column;
		this.modifier = modifier;
		this.remainder = remainder;

		if (remainder != null && remainder.column.getColumnSet() != column.getColumnSet())
			throw new RuntimeException("Columns must be from the same table");

		comparisonFactor = ((modifier & DESCENDING) != 0) ? -1 : 1;
	}

	public int getModifier() {
		return modifier;
	}

	public Order getRemainder() {
		return remainder;
	}

	public int size() {
		return remainder == null ? 1 : (remainder.size() + 1);
	}


	public Column getColumn() {
		return column;
	}


	/**
	 * Determines to which extent the ranges are supported by this ordering
	 *
	 * @param ranges Hashtable where the keys are Columns and the values are object arrays of size two containing
	 *               thr upper and lower bound
	 * @return
	 */
	public int getRangeSupport(Hashtable ranges) {

		return (ranges.get(column) != null)
				? 1 + (remainder != null ? remainder.getRangeSupport(ranges) : 0)
				: 0;
	}

	/* (non-Javadoc)
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
	public int compare(Object o1, Object o2) {

		try {
			if (o1 instanceof Integer) {
				if (o2 instanceof Integer) {
					return compare(((Integer) o1).intValue(), ((Integer) o2).intValue());
				}

				if (o2 instanceof Object[]) {
					return compare(((Integer) o1).intValue(), (Object[]) o2, 0);
				}
			} else if (o1 instanceof Object[]) {
				if (o2 instanceof Integer) {
					return -compare(((Integer) o2).intValue(), (Object[]) o1, 0);
				} else if (o2 instanceof Object[]) {
					return compare((Object[]) o1, (Object[]) o2, 0);
				}
			}
			throw new RuntimeException
					("This comparator can only compare integers denoting row numbers and Object arrays; given parameter were " + o1 + ":" + o1.getClass() + ", " + o2 + ": " + o2.getClass());
		}
		catch (SQLException e) {
			throw new RuntimeException(e.toString());
		}
	}

	public int compare(int row1, int row2) throws SQLException {
		org.kobjects.jdbc.util.AbstractTable t =
				(org.kobjects.jdbc.util.AbstractTable) column.getColumnSet();
		Comparable o1 = (Comparable) t.get(row1, column.getNumber());
		Comparable o2 = (Comparable) t.get(row2, column.getNumber());

		if (o1 == null) return o2 == null ? 0 : -comparisonFactor;
		if (o2 == null) return comparisonFactor;

		int c = o1.compareTo(o2) * comparisonFactor;
		return (c != 0 || remainder == null)
				? c
				: remainder.compare(row1, row2);
	}

	public int compare(int row1, Object[] row2, int idx) throws SQLException {
		org.kobjects.jdbc.util.AbstractTable t =
				(org.kobjects.jdbc.util.AbstractTable) column.getColumnSet();
		Comparable o1 = (Comparable) t.get(row1, column.getNumber());
		Comparable o2 = (Comparable) row2[idx];

		int c = o1.compareTo(o2) * comparisonFactor;
		return (c != 0 || remainder == null || idx + 1 == row2.length)
				? c
				: remainder.compare(row1, row2, idx + 1);
	}

	public int compare(Object[] row1, Object[] row2, int idx) throws SQLException {
		org.kobjects.jdbc.util.AbstractTable t =
				(org.kobjects.jdbc.util.AbstractTable) column.getColumnSet();
		Comparable o1 = (Comparable) row1[idx];
		Comparable o2 = (Comparable) row2[idx];

		int c = o1.compareTo(o2) * comparisonFactor;
		return (c != 0 || remainder == null || idx + 1 == row2.length)
				? c
				: remainder.compare(row1, row2, idx + 1);
	}


	public String toString() {
		return remainder == null
				? column.getName()
				: column.getName() + "," + remainder;
	}

}