package org.kobjects.jdbc;

import java.sql.*;

import org.kobjects.jdbc.sql.*;

// (C) 2002 by Stefan Haustein 
// Rolandstrasse 27, D-46045 Oberhausen, Germany
// All rights reserved.
//
// For licensing details, please refer to the file "license.txt",
// distributed with this file.

/**
 * @author Stefan Haustein
 */


public abstract class Table extends ColumnSet {


	/**
	 * Equivalent to select(null, (Expression) null, null)
	 */

	public ResultSet select() throws SQLException {
		return select(null, (Expression) null, null);
	}


	/**
	 * Equivalent to select(fields, where == null ? null : new SqlParser(this, where).parseSearchCondition(), orderBy)
	 */


	public ResultSet select(int[] fields, String where, Order orderBy) throws SQLException {

		return select(fields, where == null ? null : new SqlParser(this, where).parseSearchCondition(), orderBy);

	}


	/**
	 * <p>Returns a result set with the given fields, denoted by their column numbers, restricted
	 * by the condition expression, odered by orderBy. </p>
	 * <p/>
	 * <p><b>Note:</b> Please use getStatement().close() to close both, the result set
	 * and any statement that may have been created implicitly.</p>
	 *
	 * @param fields	Selected column numbers. null denotes all columns.
	 * @param condition Selection condition. May be null.
	 * @param orderBy   null means unordered.
	 */
	public abstract ResultSet select(int[] fields, Expression condition, Order orderBy) throws SQLException;


	/**
	 * Closes this table.
	 */

	public abstract void close() throws SQLException;


}
