package org.kobjects.jdbc.format.dbase;

import java.sql.SQLException;
import java.util.Vector;

import org.kobjects.jdbc.sql.*;
import org.kobjects.tokenizer.ParsingException;

public class DbaseParser {

	DbaseTokenizer tokenizer;
	DbaseTable table;

	DbaseParser(DbaseTable table, String expression) {
		this.table = table;
		tokenizer = new DbaseTokenizer(expression);
	}

	/**
	 * Method parse.
	 *
	 * @return Expression
	 */
	public Expression parse() throws SQLException {
		Expression result = parseExpression();
		if (!tokenizer.peek(0).equals(DbaseTokenizer.EOF))
			throw new ParsingException(tokenizer, "EOF Expected");

		return result;
	}

	Expression parseFunction(String name) throws SQLException {

		name = name.toUpperCase();

		Vector args = new Vector();
		do {
			tokenizer.read();
			args.add(parseExpression());
		}
		while (tokenizer.peek(0).equals(","));
		tokenizer.require(")");

		if (name.equals("STR"))
			return (new DbaseExpr(DbaseExpr.STR, args));
		else if (name.equals("UPPER"))
			return (new StringExpr(StringExpr.UPPER, args));
		else if (name.equals("U"))
			return (new DbaseExpr(DbaseExpr.U, args));
		else if (name.equals("LOWER"))
			return (new StringExpr(StringExpr.LOWER, args));
		else if (name.equals("TRIM") || name.equals("RTRIM"))
			return (new StringExpr(StringExpr.TRIM_TRAILING, args));
		else if (name.equals("LEN"))
			return (new StringExpr(StringExpr.CHAR_LENGTH, args));
		else if (name.equals("SPACE"))
			return (new DbaseExpr(DbaseExpr.SPACE, args));

		throw new ParsingException(tokenizer, "unknown fn: '" + name + "' args: " + args);
	}


	Expression parseExpression() throws SQLException {
		return parseAdditive();
	}

	Expression parsePrimary() throws SQLException {
		String token = tokenizer.read();
		if (tokenizer.peek().equals("("))
			return parseFunction(token);

		if (token.startsWith("\""))
			return new ConstantExpr(token.substring(1));

		if (token.startsWith("0"))
			return new ConstantExpr(
					token.indexOf('.') == -1 ? (Object) new Long(token) : new Double(token));

		return new ColumnExpr(table.getColumn(token.toLowerCase()));
	}

	Expression parseMultiplicative() throws SQLException {
		return parsePrimary();
	}

	Expression parseAdditive() throws SQLException {

		Expression p0 = parseMultiplicative();

		while (true) {
			String name = tokenizer.peek();

			if (!"+".equals(name) && !"-".equals(name))
				break;

			tokenizer.read();
			char op = name.charAt(0);
			Expression[] par = {p0, parseMultiplicative()};

			if (p0.getType() == String.class) {
				if (op != '+')
					throw new RuntimeException("- undef. for Character Strings");

				p0 = new StringExpr(StringExpr.CONCAT, par);
			} else if (isInt(p0.getType()) && isInt(par[1].getType())) {
				p0 = new IntExpr(op, par);
			} else
				p0 = new NumericExpr(op, par);
		}

		return p0;
	}

	static boolean isInt(Class c) {
		return (c == Integer.class) || (c == Long.class);
	}

}
