package org.kobjects.jdbc.format.dbase;

import java.io.*;
import java.nio.channels.*;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Locale;
import java.util.Properties;

import org.kobjects.jdbc.Column;
import org.kobjects.jdbc.util.AbstractTable;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 * @author Stefan Haustein
 *         <p/>
 *         use /home/haustein/projects/eurass/OSTKUNDE/ADRESSEN.DBF
 */
public class DbaseTable extends AbstractTable {

	String fileBase;
	String ndxBase;
	Properties properties;
	RandomAccessFile raf;
	FileChannel channel;

	RandomAccessFile dbtRaf;
	FileChannel dbtChannel;

	int headerSize;
	int[] offsets = new int[1024];
	NumberFormat[] formats;
	int fieldCount;
	int recordCount;
	int rowSize;
	byte[] rowBuf;
	int row;
	long loadTime;
	byte[] staticHeader = new byte[32];
	private static String enc;

	static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");

	/**
	 * end = pointer to first char outside scope (like in substring)
	 */

	static void setString(byte[] buf, int pos, int end, String s) {
		try {
			byte[] tmp = s.getBytes(enc);

			for (int i = 0; i < tmp.length && pos < end; i++) {
				buf[pos++] = tmp[i];
			}

			while (pos < end)
				buf[pos++] = 0;
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}

	}

	static String getString(byte[] buf, int pos, int end) {

		int len = 0;

		while (pos + len < end && buf[pos + len] != 0)
			len++;

		try {
			return new String(buf, pos, len, enc);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e.toString());
		}

	}

	static int getInt32(byte[] buf, int pos) {
		return DbaseTable.getInt16(buf, pos)
				+ 65536 * DbaseTable.getInt16(buf, pos + 2);
	}

	public DbaseTable(String file, Properties properties) throws SQLException {
		try {
			//first, look for dbi!
			this.properties = properties;
			enc = properties.getProperty("encoding");

			if (file.endsWith(".dbf")
					|| file.endsWith(".DBF")
					|| file.endsWith(".dbi")
					|| file.endsWith(".DBI"))
				fileBase = file.substring(0, file.length() - 4);
			else
				fileBase = file;

			if (new File(fileBase + ".dbi").exists()) {
				properties.load(new FileInputStream(fileBase + ".dbi"));
			}

			String dbf = properties.getProperty("dbf");

			if (dbf != null) {
				if (!new File(dbf).isAbsolute())
					dbf =
							new File(new File(fileBase).getParent(), dbf)
									.toString();

				fileBase = dbf;

				if (fileBase.endsWith(".dbf") || fileBase.endsWith(".DBF"))
					fileBase = fileBase.substring(0, fileBase.length() - 4);
			}

			System.out.println("Filebase: '" + fileBase + "'");

			File dbfFile = new File(fileBase + ".dbf");
			if (!dbfFile.exists())
				dbfFile = new File(fileBase + ".DBF");

			System.out.println("dbf-file: " + dbfFile.getAbsolutePath());

			if (!dbfFile.exists())
				throw new SQLException("dbf-file does not exist: " + fileBase + "(.dbf|.DBF)");

			raf = new RandomAccessFile(dbfFile, "rw");
			channel = raf.getChannel();

			File dbtFile = new File(fileBase + ".dbt");
			if (!dbtFile.exists())
				dbtFile = new File(fileBase + ".DBT");

			if (dbtFile.exists()) {
				dbtRaf = new RandomAccessFile(dbtFile, "rw");
				dbtChannel = dbtRaf.getChannel();
			}

			raf.seek(0);
			raf.readFully(staticHeader);
			//		recordCount = getInt32(staticHeader, 4);

			headerSize = DbaseTable.getInt16(staticHeader, 8);

			byte[] fieldDef = new byte[32];
			//int pos = 32;

			offsets[0] = 1; // first byte is deleted marker

			while (true) {
				fieldDef[0] = (byte) raf.read();
				if (fieldDef[0] == 0x0d)
					break;
				raf.readFully(fieldDef, 1, 31);

				String name = getString(fieldDef, 0, 11).toLowerCase();
				int type;
				Column column;
				int len = ((int) fieldDef[16]) & 255;
				int dec = ((int) fieldDef[17]) & 255;

				switch ((char) fieldDef[11]) {
					case 'C':
						column = new Column(name, Types.CHAR);
						column.setDisplaySize(len);
						break;
					case 'N':
						if (dec == 0) {
							column = new Column(name, Types.BIGINT);
							column.setPrecision(len);
						} else {
							column = new Column(name, Types.DOUBLE);
							column.setPrecision(len - 1);
							column.setScale(dec);
						}
						column.setDisplaySize(len);

						break;
					case 'L':
						column = new Column(name, Types.BOOLEAN);
						column.setDisplaySize(1);
						break;
					case 'D':
						column = new Column(name, Types.DATE);
						column.setDisplaySize(10);
						break;
					case 'M':
						column = new Column(name, Types.LONGVARCHAR);
						break;
					default:
						throw new RuntimeException(
								name + ": unknown type:" + fieldDef[10]);
				}
				addColumn(column);
				//pos += headerSize;
				fieldCount++;
				offsets[fieldCount] =
						offsets[fieldCount - 1] + (((int) fieldDef[16]) & 255);
			}

		} catch (IOException e) {
			throw new RuntimeException(e);
			//throw new SQLException("Error opening '"+file+";"+properties+"': "+ e.toString());
		}

		rowSize = offsets[fieldCount];
		rowBuf = new byte[offsets[fieldCount]];

		// look for index definitions

		File nb = new File(fileBase);
		ndxBase =
				new File(
						nb.getParent(),
						nb.getName().substring(0, Math.min(6, nb.getName().length())))
						.toString()
						+ "_";

		int idx = 0;
		while (true) {
			String ndxOrd = properties.getProperty("ndx-ord-" + idx);
			String ndxDef = properties.getProperty("ndx-def-" + idx);
			if (ndxDef == null && ndxOrd == null)
				break;

			openIndex(idx, ndxOrd, ndxDef);
			idx++;
		}
	}

	void openIndex(int i, String ord, String def) throws SQLException {

		String ndxi = ndxBase + (i++);
		File ndxf = new File(ndxi + ".ndx");
		if (!ndxf.exists())
			ndxf = new File(ndxi + ".NDX");

		addIndex(new NdxFile(this, ndxf, ord, def));
	}

	long getRowPos(int row) {
		return headerSize + (row - 1) * rowSize;
	}

	void loadRow(int row) throws SQLException {
		if (row == this.row && System.currentTimeMillis() - loadTime < 1000)
			return;

		this.row = row;

		try {
			raf.seek(getRowPos(row));
			raf.readFully(rowBuf);
			loadTime = System.currentTimeMillis();
		} catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	synchronized long writeMemo(String memo) throws IOException {
		FileLock dbtLock = dbtChannel.lock();

		byte[] buf = new byte[4];
		dbtRaf.seek(0);
		dbtRaf.readFully(buf);
		long adr = getInt32(buf, 0);

		//		long adr = dbtRaf.length() / 512;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		Writer w = new OutputStreamWriter(baos, enc);
		w.write(memo);
		w.flush();
		baos.write(0x01a);
		baos.write(0x01a);

		while ((baos.size() % 512) != 0) {
			baos.write(' ');
		}
		setInt32(buf, 0, (int) (adr + baos.size() / 512));
		dbtRaf.seek(0);
		dbtRaf.write(buf);

		dbtRaf.seek(adr * 512);
		dbtRaf.write(baos.toByteArray());
		dbtChannel.force(true);
		dbtLock.release();
		return adr;
	}

	public void setImpl(int row, Object[] value, boolean[] dirty)
			throws SQLException {
		try {
			FileLock lock = channel.lock(getRowPos(row), rowSize, false);
			loadTime = -1;
			loadRow(row);

			writeRow(row, value, dirty);

/*
			StringBuffer buf = new StringBuffer();

			NumberFormat df = null;

			for (int i = 0; i < value.length; i++) {
				if (!dirty[i])
					continue;

				buf.setLength(0);

				if (value[i] != null) {
					switch (getColumnType(i + 1)) {
						case Types.CHAR :
							buf.append((String) value[i]);
							while (buf.length() < getColumnDisplaySize(i + 1))
								buf.append(' ');

							break;

						case Types.LONGVARCHAR :
							buf.append("" + writeMemo((String) value[i]));
							break;

						case Types.DOUBLE :
							if (df == null) {
								df =
									(DecimalFormat) DecimalFormat.getInstance(
										Locale.US);
								df.setGroupingUsed(false);
								df.setMinimumIntegerDigits(1);
							}

							df.setMaximumFractionDigits(getScale(i + 1));
							df.setMinimumFractionDigits(getScale(i + 1));
							df.setMaximumIntegerDigits(
								getPrecision(i + 1) - getScale(i + 1) - 1);

							buf.append(
								df.format(((Number) value[i]).doubleValue()));
							break;

						case Types.BIGINT :
							buf.append(value[i].toString());
							break;

						case Types.BOOLEAN :
							buf.append(
								((Boolean) value[i]).booleanValue()
									? 'T'
									: 'F');
							break;

						case Types.DATE :
							buf.append(dateFormat.format((Date) value[i]));
							break;

						default :
							throw new RuntimeException("unsupported field type");
					}
				}

				while (buf.length() < offsets[i + 1] - offsets[i])
					buf.insert(0, ' ');

				setString(rowBuf, offsets[i], offsets[i + 1], buf.toString());
			}

			raf.seek(getRowPos(row));
			raf.write(rowBuf);
*/
			lock.release();
		} catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public synchronized String readMemo(int number) throws SQLException {

		if (number == 0)
			return null;

		try {
			dbtRaf.seek(number * 512);

			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			while (true) {
				int b = dbtRaf.read();
				if (b == 0x01a || b == -1)
					break;
				baos.write((byte) b);
			}

			return new String(baos.toByteArray(), enc);
		} catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public Object get(int row, int column) throws SQLException {

		loadRow(row);

		String value = getString(rowBuf, offsets[column - 1], offsets[column]);

		if (getColumnType(column) == Types.CHAR)
			return value;

		value = value.trim();
		if (value.length() == 0)
			return null;

		switch (getColumnType(column)) {
			case Types.LONGVARCHAR:
				return readMemo(Integer.parseInt(value));
			case Types.DOUBLE:
				return new Double(value);
			case Types.BIGINT:
				return new Long(value);
			case Types.BIT:
				return value.equals("?")
						? null
						: new Boolean("TtYy".indexOf(value) != -1);
			case Types.DATE:
				try {
					return new Date(dateFormat.parse(value).getTime());
				} catch (ParseException e) {
					//throw new SQLException(e.toString());
					e.printStackTrace();
					return null;
				}

			default:
				throw new RuntimeException("unsupported field type");
		}

	}

	public int getRowCount() throws SQLException {
		try {
			raf.seek(0);
			raf.readFully(staticHeader);
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}

		return getInt32(staticHeader, 4);
	}

	/**
	 * performs the write op but not the lock
	 */

	private void writeRow(int row, Object[] value, boolean[] dirty) throws SQLException, IOException {
		StringBuffer buf = new StringBuffer();

		NumberFormat df = null;

		for (int i = 0; i < value.length; i++) {
			if (dirty != null && !dirty[i])
				continue;

			buf.setLength(0);

			if (value[i] != null) {
				switch (getColumnType(i + 1)) {
					case Types.CHAR:
						buf.append((String) value[i]);
						while (buf.length() < getColumnDisplaySize(i + 1))
							buf.append(' ');

						break;

					case Types.LONGVARCHAR:
						buf.append("" + writeMemo((String) value[i]));
						break;

					case Types.DOUBLE:
						if (df == null) {
							df =
									(DecimalFormat) DecimalFormat.getInstance(
											Locale.US);
							df.setGroupingUsed(false);
							df.setMinimumIntegerDigits(1);
						}

						df.setMaximumFractionDigits(getScale(i + 1));
						df.setMinimumFractionDigits(getScale(i + 1));
						df.setMaximumIntegerDigits(
								getPrecision(i + 1) - getScale(i + 1) - 1);

						buf.append(
								df.format(((Number) value[i]).doubleValue()));
						break;

					case Types.BIGINT:
						buf.append(value[i].toString());
						break;

					case Types.BOOLEAN:
						buf.append(
								((Boolean) value[i]).booleanValue()
										? 'T'
										: 'F');
						break;

					case Types.DATE:
						buf.append(dateFormat.format((Date) value[i]));
						break;

					default:
						throw new RuntimeException("unsupported field type");
				}
			}

			while (buf.length() < offsets[i + 1] - offsets[i])
				buf.insert(0, ' ');

			setString(rowBuf, offsets[i], offsets[i + 1], buf.toString());
		}

		raf.seek(getRowPos(row));
		raf.write(rowBuf);

	}

	protected void addRowImpl(Object[] value) throws SQLException {

		try {
			FileLock lock = channel.lock(0, staticHeader.length, false);
			loadTime = -1;

			int row = getRowCount() + 1;

			rowBuf[0] = ' ';
			writeRow(row, value, null);

			setInt32(staticHeader, 4, row);
			raf.seek(0);
			raf.write(staticHeader);

			lock.release();
		} catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public boolean rowDeleted(int row) throws SQLException {
		loadRow(row);
		return rowBuf[0] == '*';
	}

	public void deleteRowImpl(int row) throws SQLException {
		try {
			FileLock lock = channel.lock(getRowPos(row), rowSize, false);
			loadTime = -1;

			raf.seek(getRowPos(row));
			raf.write('*');

			lock.release();
		} catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public void close() {
		// TODO Auto-generated method stub

	}

	static int getInt16(byte[] buf, int pos) {
		return (((int) buf[pos]) & 255) + 256 * (((int) buf[pos + 1]) & 255);
	}

	static void setInt32(byte[] buf, int pos, int val) {
		buf[pos] = (byte) (val & 255);
		buf[pos + 1] = (byte) ((val >> 8) & 255);
		buf[pos + 2] = (byte) ((val >> 16) & 255);
		buf[pos + 3] = (byte) ((val >> 24) & 255);
	}

	static void setInt16(byte[] buf, int pos, int val) {
		buf[pos] = (byte) (val & 255);
		buf[pos + 1] = (byte) ((val >> 8) & 255);
	}

}
