package org.kobjects.jdbc.format.dbase;

import java.io.*;
import java.sql.SQLException;
import java.util.Iterator;

import org.kobjects.jdbc.Order;
import org.kobjects.jdbc.sql.Expression;
import org.kobjects.jdbc.util.Index;
import org.kobjects.util.Csv;

/**
 * @author Stefan Haustein
 */
public class NdxFile implements Index {

	private boolean updateOldDeleted;
	DbaseTable table;
	RandomAccessFile raf;
	byte[] page = new byte[512];

	int startPage;
	int keyLength;
	int keysPerPage;
	int keyType;
	int keyRecordSize;
	//int pageCount;

	int updateRow;
	Object updateOldKey;

	String keyDefinition;
	Expression definition;
	Order order;

	NdxFile(DbaseTable table, File file, String ordDef, String keyDef)
			throws SQLException {
		this.table = table;
		try {
			raf = new RandomAccessFile(file, "rw");
			raf.seek(0);
			raf.readFully(page);
		}
		catch (IOException e) {
			throw new SQLException("IO error opening index " + file + ": " + e);
		}
		startPage = DbaseTable.getInt32(page, 0);
		//pageCount = DbaseTable.getInt32(page, 4);
		keyLength = DbaseTable.getInt16(page, 12);
		keysPerPage = DbaseTable.getInt16(page, 14);
		keyType = DbaseTable.getInt16(page, 16);
		keyRecordSize = DbaseTable.getInt16(page, 18);
		keyDefinition = DbaseTable.getString(page, 24, 512);

		if (!keyDefinition.equalsIgnoreCase(keyDef))
			System.err.println(
					"Stored def: '" + keyDefinition + "' does not match param: '" + keyDef + "'");

		//		System.out.println("keyDefinition: " + keyDefinition);

		String[] oa = Csv.decode(ordDef);
		int i = oa.length;
		while (i > 0)
			order = new Order(table.getColumn(oa[--i]), 0, order);

		definition = new DbaseParser(table, keyDef).parse();

		System.out.println("using index definition: " + definition);
	}

	public int getPageCount() {
		return DbaseTable.getInt32(page, 4);
	}

	public void setPageCount(int cnt) {
		DbaseTable.setInt32(page, 4, cnt);
	}


	void load(int index, byte[] page) {
		try {
			raf.seek(index * 512);
			raf.readFully(page, 0, 512);
		}
		catch (IOException e) {
			throw new RuntimeException(e.toString());
		}
	}

	public Expression getDefinition() {
		return definition;
	}

	public Iterator getIterator(Object[] start) throws SQLException {
		NdxIterator n = new NdxIterator(this, start);
		return n;
	}

	public void prepareUpdate(int row) throws SQLException {
		updateRow = row;
		updateOldKey = definition.eval(row, null);
	}


	public void commitUpdate() throws SQLException {
		Object updateNewKey = definition.eval(updateRow, null);
		if (!updateOldKey.equals(updateNewKey)) {
//			System.err.println("Warning: Indices are NOT updated");/*
			try {
				delete(updateOldKey, updateRow);
				insert(updateNewKey, updateRow);
			}
			catch (IOException e) {
				throw new SQLException(e.toString());
			}//*/
		}


	}


	void delete(Object key, int rnr) throws IOException {
		NdxPos pos = find(key, rnr);
		if (pos == null)
			throw new RuntimeException("Trying to delete non-existent key " + key + " with rnr " + rnr);

		pos.page.deleteRecord(pos.index);
	}


	void insert(Object key, int rnr) throws IOException {
		NdxPos pos = find(key, 0);
		pos.page.insertRecord(pos.index, rnr, key);
	}


	public NdxPos find(Object key, int rnr) {
		return new NdxPage(this, startPage, null, -1).find(key, rnr);
	}

	public void insertRow(int row) throws SQLException {
		try {
			Object key = definition.eval(row, null);
			insert(key, row);
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public void deleteRow(int row) throws SQLException {

		try {
			Object key = definition.eval(row, null);
			System.out.println("delete key: " + key);
			delete(key, row);
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
		//		throw new RuntimeException("NYI");
	}

	public Order getOrder() {
		return order;
	}


	public void save() throws IOException {
		raf.seek(0);
		raf.write(page);
	}

}


