package org.kobjects.jdbc.format.dbase;

import java.sql.SQLException;
import java.util.Iterator;

import org.kobjects.jdbc.Order;

/**
 * @author Stefan Haustein
 */

class NdxIterator implements Iterator {

	NdxFile ndx;
	NdxPage current;
	NdxPos pos;
	int ofs;
	Object next;

	NdxIterator(NdxFile ndx, Object[] start) {
		this.ndx = ndx;

		try {
			String key = getKey(start);
			NdxPos pos = ndx.find(key, 0);
			if (pos == null) {
				current = null;
				ofs = 0;
			} else {
				current = pos.page;
				ofs = pos.index;
			}
		}
		catch (SQLException e) {
			throw new RuntimeException(e.toString());
		}

	}

	/**
	 * Returns the next record numner according to the index, or -1, if the
	 * end of file has been reached.
	 */

	public boolean hasNext() {

		if (next != null)
			return true;

		int rnr;
		do {
			if (current == null)
				return false;

			// save current
			rnr = ofs < current.getCount() ? current.getRecNo(ofs) : 0;

			// advance
			ofs++;

			// end? if so, ascend to parent!

			if (ofs > current.getCount()) {
				ofs = current.parentOfs;
				current = current.parentPage;
			} else { //descend as far as possible (postorder)

				while (true) {
					NdxPage child = current.getPage(ofs);
					if (child == null)
						break;
					current = child;
					ofs = 0;
				}
			}
		}
		while (rnr == 0);

		next = new Integer(rnr);
		return true;
	}

	public Object next() {
		if (!hasNext())
			throw new RuntimeException("Read past eof");
		Object result = next;
		//	System.out.println("rnr: " + next+ " key: "+current.getKey(ofs));
		next = null;
		return result;
	}

	public String getKey(Object[] target) throws SQLException {
		Object[] t2 = new Object[ndx.table.getColumnCount()];
		Order o = ndx.order;
		for (int i = 0; i < target.length; i++) {
			t2[o.getColumn().getNumber() - 1] = target[i];
			//	System.out.println("getKey; colum: "+o.getColumn()+ " val: "+target[i]);
			o = o.getRemainder();
		}

		String seekTo = (String) ndx.definition.eval(-1, t2);

		//	System.out.println("seekTo: " + seekTo);
		return seekTo == null ? "" : seekTo;
	}

	public void remove() {
		throw new RuntimeException("Iterator remove not supported!");
	}
}
