package org.kobjects.jdbc.sql;

import java.sql.SQLException;
import java.util.Hashtable;

import org.kobjects.jdbc.Column;
import org.kobjects.util.GeneralComparator;

/**
 * @author Stefan Haustein
 *         <p/>
 *         use /home/haustein/projects/eurass/OSTKUNDE/ADRESSEN.DBF
 */
public class RelationalExpr extends Expression {


	public static final String OP_NAMES[] = {"<>", "<", ">", "<=", ">=", "="};

	/**
	 * Type constant for "not equal".
	 */
	public static final int NE = 0;


	/**
	 * Type constant for "less than".
	 */
	public static final int LT = 1;

	/**
	 * Type constant for "greater than".
	 */
	public static final int GT = 2;

	/**
	 * Type constant for "less or equal".
	 */
	public static final int LE = 3;

	/**
	 * Type constant for "greater or equal".
	 */
	public static final int GE = 4;

	/**
	 * Type constant for "equal".
	 */
	public static final int EQ = 5;


	/**
	 * Type constant for "textually equal". Results in a case-insensitive String
	 * comparison.
	 */
	public static final int EQ_TEXT = 7;


	Expression operand1;
	Expression operand2;
	int operation;

	public RelationalExpr(int operation, Expression operand1, Expression operand2) {
		this.operation = operation;
		this.operand1 = operand1;
		this.operand2 = operand2;
	}

	public static Boolean compare(int op, Object o1, Object o2) {
		if (o1 == null || o2 == null) return null;

		if (op == EQ_TEXT) {
			return b(o1.toString().equalsIgnoreCase(o2.toString()));
		}

		int c = GeneralComparator.cmp(o1, o2);
		boolean b;
		switch (op) {
			case EQ:
				b = c == 0;
				break;
			case NE:
				b = c != 0;
				break;
			case LT:
				b = c < 0;
				break;
			case GT:
				b = c > 0;
				break;
			case LE:
				b = c <= 0;
				break;
			case GE:
				b = c >= 0;
				break;
			default:
				throw new RuntimeException("Illegal Relational op const: " + op);
		}
		return b ? Boolean.TRUE : Boolean.FALSE;
	}

//	public static int compare(Object o1, Object o2) {

	//	}
	/**
	 * @see org.kobjects.jdbc.sql.Expression#eval(int)
	 */

	static Boolean b(boolean b) {
		return b ? Boolean.TRUE : Boolean.FALSE;
	}

	public Object eval(int rowNumber, Object[] rowData) throws SQLException {
		return compare(operation, operand1.eval(rowNumber, rowData), operand2.eval(rowNumber, rowData));
	}

	public void getRangeRestrictions(Hashtable result) {

		boolean invert;
		Object limit;
		Column column;

		if ((operand1 instanceof ConstantExpr) && (operand2 instanceof ColumnExpr)) {
			invert = true;
			column = ((ColumnExpr) operand2).column;
			limit = ((ConstantExpr) operand1).value;
		} else if (operand1 instanceof ColumnExpr && operand2 instanceof ConstantExpr) {
			invert = false;
			column = ((ColumnExpr) operand1).column;
			limit = ((ConstantExpr) operand2).value;
		} else
			return;

		Object[] range = new Object[2];

		switch (operation) {
			case LT:
			case LE:
				range[invert ? 0 : 1] = limit;
				break;
			case GT:
			case GE:
				range[invert ? 1 : 0] = limit;
				break;
			case EQ:
				range[0] = limit;
				range[1] = limit;
				break;
			default:
				return;
		}


		Object[] old = (Object[]) result.get(column);
		if (old != null) {
			if (range[0] == null) range[0] = old[0];
			else if (old[0] != null) { // take bigger one
				if (GeneralComparator.cmp(range[0], old[1]) > 0)
					range[0] = old[0];
			}
			if (range[1] == null) range[1] = old[1];
			else if (old[1] != null) {
				if (GeneralComparator.cmp(range[1], old[1]) < 0)
					range[1] = old[1];
			}
		}

		//	System.out.println("Range for column "+column+" from "+range[0]+" to "+range[1]);

		result.put(column, range);
	}


	public String toString(int sqlType) {
		return " ( " + operand1.toString(sqlType) + ' ' + OP_NAMES[operation] + ' ' + operand2.toString(sqlType) + " ) ";
	}

	/* (non-Javadoc)
	 * @see org.kobjects.jdbc.sql.Expression#getType()
	 */
	public Class getType() {
		return Boolean.class;
	}

}
