/*
 * Created on May 26, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package org.kobjects.jdbc.sql;

//import java.sql.SQLException;

import java.sql.SQLException;
import java.util.Vector;

import org.kobjects.jdbc.ColumnSet;
import org.kobjects.tokenizer.AbstractTokenizer;
import org.kobjects.tokenizer.ParsingException;
import org.kobjects.util.Util;

/**
 * @author haustein
 *         <p/>
 *         To change the template for this generated type comment go to
 *         Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SqlParser {

	AbstractTokenizer tokenizer;
	ColumnSet table;

	public SqlParser(ColumnSet table, SqlTokenizer tokenizer) {
		this.table = table;
		this.tokenizer = tokenizer;
	}

	public SqlParser(ColumnSet table, String expr) {
		this(table, new SqlTokenizer(expr));
	}

/*
	public Expression parse() throws SQLException{
		
		return 
	}
*/

	public Expression parseSearchCondition() {

		Expression result = parseBooleanTerm();

		if (tokenizer.peek(0).equalsIgnoreCase("OR")) {
			Vector children = new Vector();
			children.addElement(result);
			do {
				tokenizer.read();
				children.addElement(parseBooleanTerm());
			}
			while (tokenizer.peek().equalsIgnoreCase("OR"));
			result = new BooleanExpr(BooleanExpr.OR, children);
		}

		return result;
	}

	/**
	 * @return
	 */
	private Expression parseBooleanTerm() {

		Expression result = parseBooleanFactor();

		if (tokenizer.peek(0).equalsIgnoreCase("AND")) {
			Vector children = new Vector();
			children.addElement(result);
			do {
				tokenizer.read();
				children.addElement(parseBooleanFactor());
			}
			while (tokenizer.peek().equalsIgnoreCase("AND"));
			result = new BooleanExpr(BooleanExpr.AND, children);
		}

		return result;
	}


	private Expression parseBooleanFactor() {
		return tokenizer.peek(0).equalsIgnoreCase("NOT")
				? new NotExpr(parseBooleanTest())
				: parseBooleanTest();
	}

	private Expression parseBooleanTest() {
		Expression result = parseBooleanPrimary();
		if (tokenizer.peek(0).equalsIgnoreCase("IS")) {
			Vector par = new Vector();
			par.addElement(result);
			tokenizer.read();
			boolean not = tokenizer.peek(0).equalsIgnoreCase("NOT");
			if (not) {
				tokenizer.read();
			}
			par.addElement(parseTruthValue());
			result = new BooleanExpr(BooleanExpr.IS, par);
			if (not) result = new NotExpr(result);
		}
		return result;
	}


	private Expression parseTruthValue() {
		String val = tokenizer.read();
		Object result;
		if (val.equalsIgnoreCase("TRUE")) result = Boolean.TRUE;
		else if (val.equalsIgnoreCase("FALSE")) result = Boolean.FALSE;
		else if (val.equalsIgnoreCase("UNKNOWN")) result = null;
		else throw new ParsingException(tokenizer, "Illegal truth value: " + val);

		return new ConstantExpr(result);
	}


	private Expression parseBooleanPrimary() {
		Expression result;

		if (tokenizer.peek(0).equals("(")) {
			tokenizer.require("(");
			result = parseSearchCondition();
			tokenizer.require(")");
		} else
			result = parsePredicate();

		return result;
	}


	private Expression parsePredicate() {
		Expression result;

		Expression par1 = parseValueExpression();

		boolean not = tokenizer.peek(0).equalsIgnoreCase("NOT");
		if (not)
			tokenizer.read();

		String next = tokenizer.read();

		if (next.equalsIgnoreCase("LIKE")) {
			Expression pattern = parseValueExpression();
			Expression escape = null;
			if (tokenizer.peek(0).equalsIgnoreCase("ESCAPE")) {
				tokenizer.read();
				escape = parseValueExpression();
			}
			result = new LikeExpr(par1, pattern, escape);
		} else {
			Expression par2 = parseValueExpression();

			int opCode = Util.indexOf(RelationalExpr.OP_NAMES, next);

			if (opCode == -1)
				throw new ParsingException(tokenizer, "illegal predicate: '" + next + "'");

			result = new RelationalExpr(opCode, par1, par2);
		}
		return result;
	}


	public Expression parseValueFunction(String name) {
		tokenizer.require("(");
		Vector parV = new Vector();

		while (!tokenizer.peek().equals(")")) {
			parV.add(parseValueExpression());
			if (tokenizer.peek().equals(",") || tokenizer.peek().equalsIgnoreCase("in"))
				tokenizer.read();
			else if (!tokenizer.peek().equals(")"))
				throw new ParsingException(tokenizer, "',' (perhaps 'in') or ')' expected. next actual token is: " + tokenizer.peek());
		}

		tokenizer.require(")");

		Class type = ((Expression) parV.elementAt(0)).getType();

		if (type == String.class) {
			return new StringExpr(name, parV);
		} else throw new RuntimeException("unknown SQL function type: " + type + " (function name: '" + name + "'");
	}


	public Expression parseValueExpression() {
		// TODO incomplete!
		// please note that functions are handled in primary, too

		return parseValueExpressionPrimary();
	}

	public Expression parseValueExpressionPrimary() {

		String token = tokenizer.read();
		if (token.startsWith("'"))
			return new ConstantExpr(token.substring(1));

		if (token.startsWith("0"))
			return token.indexOf('.') == -1
					? new ConstantExpr(new Integer(token))
					: new ConstantExpr(new Double(token));

		if (token.equalsIgnoreCase("true") || token.equalsIgnoreCase("false"))
			return new ConstantExpr(new Boolean(token.toLowerCase()));

		if (tokenizer.peek().equals("("))
			return parseValueFunction(token);


		try {
			return new ColumnExpr(table.getColumn(table.findColumn(token)));
		}
		catch (SQLException e) {
			throw new ParsingException(tokenizer, e.toString());
		}
	}

}
