/*
 * Created on Oct 12, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.kobjects.jdbc.sql;

import java.sql.SQLException;
import java.util.Vector;

import org.kobjects.util.Util;

/**
 * Operations on character strings (SQL92, 4.2.2) Rule of thumb: First param
 * is a String
 */

public class StringExpr extends SimpleExpr {

	public static final int UPPER = 0;
	public static final int LOWER = 1;
	public static final int POSITION = 2;
	public static final int CHAR_LENGTH = 3;
	public static final int CONCAT = 101;
	public static final int TRIM_LEADING = 103;
	public static final int TRIM_TRAILING = 104;
	public static final int TRIM_BOTH = 105;

	int type;

	static String[] NAMES = {"UPPER", "LOWER", "POSITION", "CHAR_LENGTH"};

	public StringExpr(int type, Expression[] args) {
		super(args);
		this.type = type;
	}


	public StringExpr(int type, Vector args) {
		super(args);
		this.type = type;
	}

	public StringExpr(String name, Vector args) {
		this(Util.indexOf(NAMES, name.toUpperCase()), args);
	}


	public static String trimTrailing(String source, char c) {
		int p = source.length() - 1;
		while (p >= 0 && source.charAt(p) == c)
			p--;

		return p == source.length() ? source : source.substring(0, p + 1);
	}

	public static String trimLeading(String source, char c) {
		int len = source.length();
		int p = 0;
		while (p < len && source.charAt(p) == c)
			p++;

		return p == 0 ? source : source.substring(p);
	}


	protected Object eval(Object[] args) throws SQLException {
		String self = ((String) args[0]);


		switch (type) {
			case UPPER:
				return self.toUpperCase();
			case LOWER:
				return self.toLowerCase();
			case TRIM_LEADING:
				return trimLeading(self, ' ');
			case TRIM_TRAILING:
				return trimTrailing(self, ' ');
			case TRIM_BOTH:
				return trimTrailing(trimLeading(self, ' '), ' ');
			case CHAR_LENGTH:
				return new Long(self.length());
			case CONCAT:
				return self + args[1];
			case POSITION:
				return new Long(((String) args[1]).indexOf(self) + 1);
			default:
				throw new RuntimeException("illegal type code:" + type);
		}
	}

	public Class getType() {
		switch (type) {
			case UPPER:
			case LOWER:
			case TRIM_LEADING:
			case TRIM_TRAILING:
			case TRIM_BOTH:
			case CONCAT:
				return String.class;
			case CHAR_LENGTH:
			case POSITION:
				return Long.class;
			default:
				throw new RuntimeException("illegal type code:" + type);
		}
	}

	public String toString(int sqlType) {
		switch (type) {
			case POSITION:
				return sqlType == ORACLE ? "INSTR(" + argExpr[1].toString(sqlType) + ", " + argExpr[0].toString(sqlType) + ")" : "POSITION ( " + argExpr[0] + " in " + argExpr[1] + " ) ";
			default:
				return toString(sqlType, NAMES[type]);
		}

	}

}

