package org.kobjects.jdbc.util;

import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.*;
import java.util.Calendar;
import java.util.Map;

import org.kobjects.jdbc.ColumnSet;

/**
 * Maps all methods get and update methods to getObject/updateObject
 */

public abstract class AbstractResultSet
		implements ResultSet {

	private ColumnSet columns = new ColumnSet();
	private Statement statement = null;


	public ColumnSet getColumnSet() {
		return columns;
	}

	public String getString(int columnIndex) throws SQLException {
		Object o = getObject(columnIndex);
		return o == null ? null : o.toString();
	}

	public boolean getBoolean(int columnIndex) throws SQLException {
		Boolean b = (Boolean) getObject(columnIndex);
		return b == null ? false : b.booleanValue();
	}

	public byte getByte(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.byteValue();
	}

	public short getShort(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.shortValue();
	}

	public int getInt(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.intValue();
	}

	public long getLong(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.longValue();
	}

	public float getFloat(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.floatValue();
	}

	public double getDouble(int columnIndex) throws SQLException {
		Number n = (Number) getObject(columnIndex);
		return n == null ? 0 : n.doubleValue();
	}

	public BigDecimal getBigDecimal(int columnIndex, int scale)
			throws SQLException {
		return (BigDecimal) getObject(columnIndex);
	}

	public byte[] getBytes(int columnIndex) throws SQLException {
		return (byte[]) getObject(columnIndex);
	}

	public Date getDate(int columnIndex) throws SQLException {
		return (Date) getObject(columnIndex);
	}

	public Time getTime(int columnIndex) throws SQLException {
		return (Time) getObject(columnIndex);
	}

	public Timestamp getTimestamp(int columnIndex) throws SQLException {
		return (Timestamp) getObject(columnIndex);
	}

	public InputStream getAsciiStream(int columnIndex) throws SQLException {
		throw new RuntimeException("not supported");
	}

	public InputStream getUnicodeStream(int columnIndex) throws SQLException {
		throw new RuntimeException("not supported");
	}

	public InputStream getBinaryStream(int columnIndex) throws SQLException {
		Blob blob = getBlob(columnIndex);
		return blob == null
				? null
				: getBlob(columnIndex).getBinaryStream();
	}

	public String getString(String columnName) throws SQLException {
		return getString(findColumn(columnName));
	}

	public boolean getBoolean(String columnName) throws SQLException {
		return getBoolean(findColumn(columnName));
	}

	public byte getByte(String columnName) throws SQLException {
		return getByte(findColumn(columnName));
	}

	public short getShort(String columnName) throws SQLException {
		return getShort(findColumn(columnName));
	}

	public int getInt(String columnName) throws SQLException {
		return getInt(findColumn(columnName));
	}

	public long getLong(String columnName) throws SQLException {
		return getLong(findColumn(columnName));
	}

	public float getFloat(String columnName) throws SQLException {
		return getFloat(findColumn(columnName));
	}

	public double getDouble(String columnName) throws SQLException {
		return getDouble(findColumn(columnName));
	}

	public BigDecimal getBigDecimal(String columnName, int scale)
			throws SQLException {
		return getBigDecimal(findColumn(columnName), scale);
	}

	public byte[] getBytes(String columnName) throws SQLException {
		return getBytes(findColumn(columnName));
	}

	public Date getDate(String columnName) throws SQLException {
		return getDate(findColumn(columnName));
	}

	public Time getTime(String columnName) throws SQLException {
		return getTime(findColumn(columnName));
	}

	public Timestamp getTimestamp(String columnName) throws SQLException {
		return getTimestamp(findColumn(columnName));
	}

	public InputStream getAsciiStream(String columnName) throws SQLException {
		return getAsciiStream(findColumn(columnName));
	}

	public InputStream getUnicodeStream(String columnName)
			throws SQLException {
		return getUnicodeStream(findColumn(columnName));
	}

	public InputStream getBinaryStream(String columnName) throws SQLException {
		return getBinaryStream(findColumn(columnName));
	}

	public SQLWarning getWarnings() throws SQLException {
		return null;
	}

	public void clearWarnings() throws SQLException {
	}

	public String getCursorName() throws SQLException {
		return null;
	}

	public ResultSetMetaData getMetaData() throws SQLException {
		return columns;
	}

	public Object getObject(String columnName) throws SQLException {
		return getObject(findColumn(columnName));
	}

	public int findColumn(String columnName) throws SQLException {
		return columns.findColumn(columnName);
	}

	public Reader getCharacterStream(int columnIndex) throws SQLException {
		return getMetaData().getColumnType(columnIndex) == Types.CLOB
				? getClob(columnIndex).getCharacterStream()
				: new StringReader(getString(columnIndex));
	}

	public Reader getCharacterStream(String columnName) throws SQLException {
		return getCharacterStream(findColumn(columnName));
	}

	public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
		return (BigDecimal) getObject(columnIndex);
	}

	public BigDecimal getBigDecimal(String columnName) throws SQLException {
		return getBigDecimal(findColumn(columnName));
	}

	public boolean isBeforeFirst() throws SQLException {
		return getRow() < 1;
	}

	public boolean isFirst() throws SQLException {
		return getRow() == 1 && isAfterLast();
	}

	public void beforeFirst() throws SQLException {
		absolute(0);
	}

	public boolean first() throws SQLException {
		return absolute(1);
	}


	public boolean relative(int rows) throws SQLException {
		return absolute(getRow() + rows);
	}

	public boolean previous() throws SQLException {
		return absolute(getRow() - 1);
	}


	public void setFetchDirection(int direction) throws SQLException {
	}


	public int getFetchDirection() throws SQLException {
		return 0;
	}


	public void setFetchSize(int rows) throws SQLException {
	}


	public int getFetchSize() throws SQLException {
		return 0;
	}


	public int getType() throws SQLException {
		return 0;
	}


	public int getConcurrency() throws SQLException {
		return 0;
	}

	public void updateNull(int columnIndex) throws SQLException {
		updateObject(columnIndex, null);
	}


	public void updateBoolean(int columnIndex, boolean x) throws SQLException {
		updateObject(columnIndex, new Boolean(x));
	}

	public void updateByte(int columnIndex, byte x) throws SQLException {
		updateObject(columnIndex, new Byte(x));
	}

	public void updateShort(int columnIndex, short x) throws SQLException {
		updateObject(columnIndex, new Short(x));
	}

	public void updateInt(int columnIndex, int x) throws SQLException {
		updateObject(columnIndex, new Integer(x));
	}

	public void updateLong(int columnIndex, long x) throws SQLException {
		updateObject(columnIndex, new Long(x));
	}

	public void updateFloat(int columnIndex, float x) throws SQLException {
		updateObject(columnIndex, new Float(x));
	}

	public void updateDouble(int columnIndex, double x) throws SQLException {
		updateObject(columnIndex, new Double(x));
	}

	public void updateBigDecimal(int columnIndex, BigDecimal x)
			throws SQLException {
		updateObject(columnIndex, x);
	}

	public void updateString(int columnIndex, String x) throws SQLException {
		updateObject(columnIndex, x);
	}

	public void updateBytes(int columnIndex, byte[] x) throws SQLException {
		updateObject(columnIndex, x);
	}

	public void updateDate(int columnIndex, Date x) throws SQLException {
		updateObject(columnIndex, x);
	}

	public void updateTime(int columnIndex, Time x) throws SQLException {
		updateObject(columnIndex, x);
	}

	public void updateTimestamp(int columnIndex, Timestamp x)
			throws SQLException {
		updateObject(columnIndex, x);
	}


	public void updateAsciiStream(int columnIndex, InputStream x, int length)
			throws SQLException {
		throw new SQLException("Not Supported");
	}


	public void updateBinaryStream(int columnIndex, InputStream x, int length)
			throws SQLException {

		byte[] buf = new byte[length];

		int pos = 0;
		try {
			while (pos < length) {
				if (x.read(buf, pos, length - pos) <= 0)
					throw new SQLException("Unexpected EOF");
			}
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
		updateObject(columnIndex, buf);
	}


	public void updateCharacterStream(int columnIndex, Reader x, int length)
			throws SQLException {

		char[] buf = new char[length];
		int pos = 0;
		try {
			while (pos < length) {
				if (x.read(buf, pos, length - pos) <= 0)
					throw new SQLException("Unexpected EOF");
			}
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
		updateObject(columnIndex, new String(buf));
	}

	public void updateObject(int columnIndex, Object x, int scale)
			throws SQLException {
		throw new SQLException("Not Supported");
	}

	public void updateNull(String columnName) throws SQLException {
		updateNull(findColumn(columnName));
	}

	public void updateBoolean(String columnName, boolean x)
			throws SQLException {
		updateBoolean(findColumn(columnName), x);
	}

	public void updateByte(String columnName, byte x) throws SQLException {
		updateByte(columnName, x);
	}

	public void updateShort(String columnName, short x) throws SQLException {
		updateShort(findColumn(columnName), x);
	}

	public void updateInt(String columnName, int x) throws SQLException {
		updateInt(findColumn(columnName), x);
	}

	public void updateLong(String columnName, long x) throws SQLException {
		updateLong(findColumn(columnName), x);
	}

	public void updateFloat(String columnName, float x) throws SQLException {
		updateFloat(findColumn(columnName), x);
	}

	public void updateDouble(String columnName, double x) throws SQLException {
		updateDouble(findColumn(columnName), x);

	}

	public void updateBigDecimal(String columnName, BigDecimal x)
			throws SQLException {
		updateBigDecimal(findColumn(columnName), x);

	}

	public void updateString(String columnName, String x) throws SQLException {
		updateString(findColumn(columnName), x);
	}

	public void updateBytes(String columnName, byte[] x) throws SQLException {
		updateBytes(findColumn(columnName), x);

	}

	public void updateDate(String columnName, Date x) throws SQLException {
		updateDate(findColumn(columnName), x);

	}

	public void updateTime(String columnName, Time x) throws SQLException {
		updateTime(findColumn(columnName), x);
	}

	public void updateTimestamp(String columnName, Timestamp x)
			throws SQLException {
		updateTimestamp(findColumn(columnName), x);
	}

	public void updateAsciiStream(String columnName, InputStream x, int length)
			throws SQLException {
		updateAsciiStream(findColumn(columnName), x, length);

	}

	public void updateBinaryStream(
			String columnName,
			InputStream x,
			int length)
			throws SQLException {
		updateBinaryStream(findColumn(columnName), x, length);
	}

	public void updateCharacterStream(
			String columnName,
			Reader reader,
			int length)
			throws SQLException {
		updateCharacterStream(findColumn(columnName), reader, length);
	}

	public void updateObject(String columnName, Object x, int scale)
			throws SQLException {
		updateObject(findColumn(columnName), x);
	}

	public void updateObject(String columnName, Object x) throws SQLException {
		updateObject(findColumn(columnName), x);
	}

	public void cancelRowUpdates() throws SQLException {
		throw new SQLException("not supported!");
	}

	public Statement getStatement() throws SQLException {
		if (statement == null)
			statement = new DummyStatement(this);

		return statement;
	}


	public Object getObject(int i, Map map) throws SQLException {
		throw new SQLException("not supported!");

	}

	public Ref getRef(int i) throws SQLException {
		throw new SQLException("not supported!");
	}

	public Blob getBlob(int i) throws SQLException {
		return (Blob) getObject(i);
	}

	public Clob getClob(int i) throws SQLException {
		return (Clob) getObject(i);
	}

	public Array getArray(int i) throws SQLException {
		throw new SQLException("not supported!");
	}

	public Object getObject(String colName, Map map) throws SQLException {
		return getObject(findColumn(colName), map);
	}


	public Ref getRef(String colName) throws SQLException {
		throw new SQLException("not supported!");
	}


	public Blob getBlob(String colName) throws SQLException {
		return getBlob(findColumn(colName));
	}


	public Clob getClob(String colName) throws SQLException {
		return getClob(findColumn(colName));
	}


	public Array getArray(String colName) throws SQLException {
		return getArray(findColumn(colName));
	}

	public Date getDate(int columnIndex, Calendar cal) throws SQLException {
		throw new SQLException("not supported!");
	}

	public Date getDate(String columnName, Calendar cal) throws SQLException {
		return getDate(findColumn(columnName), cal);
	}

	public Time getTime(int columnIndex, Calendar cal) throws SQLException {
		throw new SQLException("not supported!");
	}

	public Time getTime(String columnName, Calendar cal) throws SQLException {
		return getTime(findColumn(columnName), cal);
	}

	public Timestamp getTimestamp(int columnIndex, Calendar cal)
			throws SQLException {
		throw new SQLException("not supported!");
	}

	public Timestamp getTimestamp(String columnName, Calendar cal)
			throws SQLException {
		return getTimestamp(findColumn(columnName), cal);
	}

	public URL getURL(int columnIndex) throws SQLException {
		throw new SQLException("not supported!");
	}

	public URL getURL(String columnName) throws SQLException {
		return getURL(findColumn(columnName));
	}

	public void updateRef(int columnIndex, Ref x) throws SQLException {
		throw new SQLException("not supported!");
	}


	public void updateRef(String columnName, Ref x) throws SQLException {
		updateRef(findColumn(columnName), x);
	}


	public void updateBlob(int columnIndex, Blob x) throws SQLException {
		updateObject(columnIndex, x);
	}


	public void updateBlob(String columnName, Blob x) throws SQLException {
		updateBlob(findColumn(columnName), x);
	}


	public void updateClob(int columnIndex, Clob x) throws SQLException {
		throw new SQLException("not supported!");
	}

	public void updateClob(String columnName, Clob x) throws SQLException {
	}

	public void updateArray(int columnIndex, Array x) throws SQLException {
		throw new SQLException("not supported!");
	}

	public void updateArray(String columnName, Array x) throws SQLException {
		updateArray(findColumn(columnName), x);
	}

	public boolean last() throws SQLException {
		while (!isLast() && next()) ;

		return !isAfterLast();
	}

	public void afterLast() throws SQLException {
		while (next()) ;
	}

}
