package org.kobjects.jdbc.util;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import org.kobjects.jdbc.*;
import org.kobjects.jdbc.sql.Expression;


/**
 * @author Stefan Haustein
 */


public abstract class AbstractTable extends Table {

	Vector indices = new Vector();

	/**
	 * Call this method if an index shall be kept consistent
	 */

	public void addIndex(Index index) {
		indices.addElement(index);
	}


	public Index getIndex(int i) {
		return (Index) indices.elementAt(i);
	}

	public int getIndexCount() {
		return indices.size();
	}


	public void set(int row, Object[] values, boolean[] dirty) throws SQLException {
		Object[] oldKeys = new Object[getIndexCount()];

		for (int i = 0; i < oldKeys.length; i++)
			getIndex(i).prepareUpdate(row);

		setImpl(row, values, dirty);

		for (int i = 0; i < oldKeys.length; i++)
			getIndex(i).commitUpdate();
	}

	protected abstract void setImpl(int row, Object[] values, boolean[] dirty) throws SQLException;


	/**
	 * Row number -1 indicates insert row
	 */

	public Object createLob(int row, int column) throws SQLException {
		throw new SQLException("Lob-objects not supported by " + getClass());
	}


	public abstract Object get(int row, int column) throws SQLException;

	public abstract int getRowCount() throws SQLException;

	public void addRow(Object[] row) throws SQLException {
		addRowImpl(row);

		for (int i = 0; i < getIndexCount(); i++) {
			getIndex(i).insertRow(getRowCount());
		}
	}

	protected abstract void addRowImpl(Object[] row) throws SQLException;

	public abstract boolean rowDeleted(int row) throws SQLException;


	public ResultSet select() throws SQLException {
		seal();
		return select(null, (Expression) null, null);
	}

	public ResultSet select(int[] fields, Expression condition, Order orderBy) throws SQLException {
		return new TableResultSet(this, fields, condition, orderBy);
	}


	/**
	 * Method deleteRow.
	 *
	 * @param row
	 */
	public void deleteRow(int row) throws SQLException {

		Object[] oldKeys = new Object[getIndexCount()];

		for (int i = 0; i < oldKeys.length; i++)
			getIndex(i).deleteRow(row);

		deleteRowImpl(row);
	}

	/*
		void fillOrder(int row, Order order, Object[] result) throws SQLException {
			for(int i =0; i < result.length; i++){
				result[i] = get(row, order.getColumn().getNumber(), false);
				order = order.getRemainder();
			}
		}

	  */
	protected abstract void deleteRowImpl(int row) throws SQLException;


}
