package org.kobjects.jdbc.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.sql.Blob;
import java.sql.SQLException;

//import org.kobjects.util.Util;

/**
 * @author Stefan Haustein
 */
public class FileBlob implements Blob {

	public File file;

	public FileBlob(File file) {
		this.file = file;
	}

	/**
	 * @see java.sql.Blob#length()
	 */
	public long length() throws SQLException {
		return file.length();
	}

	/**
	 * @see java.sql.Blob#getBytes(long, int)
	 */
	public byte[] getBytes(long pos, int length) throws SQLException {
		byte[] data = new byte[length];

		try {
			RandomAccessFile raf = new RandomAccessFile(file, "r");

			raf.seek(pos - 1);
			raf.readFully(data);
			raf.close();
			return data;
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	/**
	 * @see java.sql.Blob#getBinaryStream()
	 */
	public InputStream getBinaryStream() throws SQLException {
		try {
			return new FileInputStream(file);
		}
		catch (FileNotFoundException e) {
			throw new SQLException(e.toString());
		}
	}

	/**
	 * @see java.sql.Blob#position(byte[], long)
	 */
	public long position(byte[] pattern, long start) throws SQLException {
		throw new RuntimeException("NYI");
	}

	/**
	 * @see java.sql.Blob#position(java.sql.Blob, long)
	 */
	public long position(Blob pattern, long start) throws SQLException {
		throw new RuntimeException("NYI");
	}

	/**
	 * @see java.sql.Blob#setBytes(long, byte[])
	 */
	public int setBytes(long pos, byte[] bytes) throws SQLException {
		return setBytes(pos, bytes, 0, bytes.length);
	}

	/**
	 * @see java.sql.Blob#setBytes(long, byte[], int, int)
	 */
	public int setBytes(long pos, byte[] bytes, int offset, int len)
			throws SQLException {

		try {
			RandomAccessFile raf = new RandomAccessFile(file, "rw");
			raf.seek(pos - 1);
			raf.write(bytes, offset, len);
			raf.close();
			return len;
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	/**
	 * @see java.sql.Blob#setBinaryStream(long)
	 */
	public OutputStream setBinaryStream(long pos) throws SQLException {
		truncate(pos - 1);
		try {
			return new FileOutputStream(file, true);
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	/**
	 * @see java.sql.Blob#truncate(long)
	 */
	public void truncate(long len) throws SQLException {
		RandomAccessFile raf;
		try {
			raf = new RandomAccessFile(file, "rw");

			raf.setLength(len);
			raf.close();
		}
		catch (IOException e) {
			throw new SQLException(e.toString());
		}
	}

	public void free() throws SQLException {
	}

	public InputStream getBinaryStream(long pos, long length) throws SQLException {
		return null;
	}
}
